package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceAudiencePackageGet;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceAudiencePackageGetMapper;
import cn.com.ctop.job.bytedance.data.mapper.DataMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceAudiencePackageGetService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxl.job.core.context.XxlJobHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 获取定向包
 *
 * @author jeecg-boot
 * 2021-10-21
 * @version V1.0
 */
@Service
public class BytedanceAudiencePackageGetServiceImpl extends ServiceImpl<BytedanceAudiencePackageGetMapper, BytedanceAudiencePackageGet> implements IBytedanceAudiencePackageGetService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;

    @Autowired
    DataMapper dataMapper;



    /**
     * -------------------------------------------------------------------------获取定向包
     */
    @Override
    public void getAudiencePackageGetReport(OauthToken token) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("advertiser_id", token.getAccountId());
        jsonObject.put("page_size", 100);
        getAudiencePackageGetReportByPage(token, jsonObject, 1);
    }

    /**
     * 获取定向包
     */
    private void getAudiencePackageGetReportByPage(OauthToken token, JSONObject conditions, Integer page) {
        conditions.put("page", page);
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        JSONObject jsonObject = JSONObject.parseObject(JSON.toJSONString(conditions, config));
        JSONObject getObject = getAudiencePackageGetStat(token, jsonObject);
        if (null == getObject) {
            XxlJobHelper.log("头条代理商数据获取异常");
            return;
        }
        Integer code = getObject.getInteger("code");
        String message = getObject.getString("message");
        if (null == code || code != 0) {
            XxlJobHelper.log("头条代理商数据获取异常：{}", message);
            return;
        }
        JSONArray dataArray = getObject.getJSONObject("data").getJSONArray("audience_packages");
        if (null == dataArray || dataArray.isEmpty()) {
            return;
        }

        for (int i = 0; i < dataArray.size(); i++) {
            JSONObject data = dataArray.getJSONObject(i);
            BytedanceAudiencePackageGet report = JSONObject.parseObject(data.toJSONString(), BytedanceAudiencePackageGet.class);
            BytedanceAudiencePackageGet audiencePackageByPrimaryKey = dataMapper.getAudiencePackageByPrimaryKey(report.getAdvertiserId(), report.getAudiencePackageId());
            if (audiencePackageByPrimaryKey != null && audiencePackageByPrimaryKey.getAdvertiserId() != null) {
                UpdateWrapper<BytedanceAudiencePackageGet> updateWrapper = new UpdateWrapper<>();
                updateWrapper.eq("advertiser_id", report.getAdvertiserId())
                        .eq("audience_package_id", report.getAudiencePackageId());
                this.update(report, updateWrapper);
            } else {
                this.save(report);
            }
        }
        page++;
        int totalPage = getObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (page <= totalPage) {
            getAudiencePackageGetReportByPage(token, conditions, page);
        }
    }

    /**
     * 获取定向包报表信息
     *
     * @param token
     * @param conditions
     * @return
     */
    public JSONObject getAudiencePackageGetStat(OauthToken token, JSONObject conditions) {
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_AUDIENCE_PACKAGE_GET;
        return HttpUtils.bytedanceGetRequest(token.getAccessToken(), url, conditions);
    }

}
