package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceCampaignGet;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceCampaignGetMapper;
import cn.com.ctop.job.bytedance.data.mapper.DataMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCampaignGetService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 获取定向包
 *
 * @author jeecg-boot
 * 2021-10-21
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceCampaignGetServiceImpl extends ServiceImpl<BytedanceCampaignGetMapper, BytedanceCampaignGet> implements IBytedanceCampaignGetService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Autowired
    DataMapper dataMapper;

    @Override
    public void getCampaignData(OauthToken oauthToken, int pageNumber, int pageSize) {

        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_CAMPAIGN_GET;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("page", pageNumber);
        param.put("page_size", pageSize);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告组接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }

        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            BytedanceCampaignGet report = JSONObject.parseObject(dataObject.toJSONString(), BytedanceCampaignGet.class);
            report.setAdvertiserId(oauthToken.getAccountId());
            report.setCampaignId((Long) dataObject.get("id"));
            report.setCampaignName((String) dataObject.get("name"));
            BytedanceCampaignGet campaignByPrimaryKey = dataMapper.getCampaignByPrimaryKey(report.getAdvertiserId(), report.getCampaignId());
            if (campaignByPrimaryKey != null && campaignByPrimaryKey.getAdvertiserId() != null) {
                UpdateWrapper<BytedanceCampaignGet> updateWrapper = new UpdateWrapper<>();
                updateWrapper.eq("advertiser_id", report.getAdvertiserId())
                        .eq("campaign_id", report.getCampaignId());
                this.update(report, updateWrapper);
            } else {
                this.save(report);
            }

        }
        pageNumber++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (pageNumber <= totalPage) {
            getCampaignData(oauthToken, pageNumber, 1000);
        }
    }

}
