package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceCreativeGet;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceCreativeGetMapper;
import cn.com.ctop.job.bytedance.data.mapper.DataMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceCreativeGetService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 创意信息
 *
 * @author jeecg-boot
 * 2021-10-22
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceCreativeGetServiceImpl extends ServiceImpl<BytedanceCreativeGetMapper, BytedanceCreativeGet> implements IBytedanceCreativeGetService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Autowired
    DataMapper dataMapper;

    @Override
    public void getCreativeData(OauthToken oauthToken, int page, int pageSize) {
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_CREATIVE_GET;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("page", page);
        param.put("page_size", pageSize);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取创意接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }

        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            BytedanceCreativeGet report = JSONObject.parseObject(dataObject.toJSONString(), BytedanceCreativeGet.class);

            BytedanceCreativeGet bytedanceCreativeGet = dataMapper.getCreativeByPrimaryKey(report.getAdvertiserId(), report.getAdId(), report.getCreativeId());
            if (bytedanceCreativeGet != null && bytedanceCreativeGet.getAdvertiserId() != null) {
                UpdateWrapper<BytedanceCreativeGet> updateWrapper = new UpdateWrapper<>();
                updateWrapper.eq("advertiser_id", report.getAdvertiserId())
                        .eq("ad_id", report.getAdId())
                        .eq("creative_id", report.getCreativeId());
                this.update(report, updateWrapper);
            } else {
                this.save(report);
            }

        }
        page++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (page <= totalPage) {
            getCreativeData(oauthToken, page, 1000);
        }

    }
}
