package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceFileVideoGet;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceFileVideoGetMapper;
import cn.com.ctop.job.bytedance.data.mapper.DataMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceFileVideoGetService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 头条-视频素材
 *
 * @author jeecg-boot
 * 2021-10-22
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceFileVideoGetServiceImpl extends ServiceImpl<BytedanceFileVideoGetMapper, BytedanceFileVideoGet> implements IBytedanceFileVideoGetService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Autowired
    DataMapper dataMapper;

    @Override
    public void getFileVideoData(OauthToken oauthToken, int page, int pageSize) {
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_FILE_VIDEO_GET;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("page", page);
        param.put("page_size", pageSize);
        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取视频素材接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }

        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
          try {
              BytedanceFileVideoGet report = JSONObject.parseObject(dataObject.toJSONString(), BytedanceFileVideoGet.class);
              report.setAdvertiserId(oauthToken.getAccountId());

              BytedanceFileVideoGet bytedanceCreativeGet = dataMapper.getFileVideoGetByPrimaryKey(report.getAdvertiserId(), report.getId());

              report.setVideoUrl("https://aweme.snssdk.com/aweme/v1/playwm/?video_id=" + report.getId() + "&line=0");
              if (bytedanceCreativeGet != null && bytedanceCreativeGet.getAdvertiserId() != null) {
                  UpdateWrapper<BytedanceFileVideoGet> updateWrapper = new UpdateWrapper<>();
                  updateWrapper.eq("advertiser_id", report.getAdvertiserId())
                          .eq("id", report.getId());
                  this.update(report, updateWrapper);
              }  else {
                  this.save(report);
              }
          }catch (Exception e){
              log.info("错误:{}",e.toString());
          }

        }
        page++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (page <= totalPage) {
            getFileVideoData(oauthToken, page, 100);
        }
    }
}
