package cn.com.ctop.job.bytedance.data.service.impl;

import cn.com.ctop.job.bytedance.data.constant.BytedanceConstant;
import cn.com.ctop.job.bytedance.data.entity.BytedanceMaterialReportDaily;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.mapper.BytedanceMaterialReportDailyMapper;
import cn.com.ctop.job.bytedance.data.mapper.DataMapper;
import cn.com.ctop.job.bytedance.data.service.IBytedanceMaterialReportDailyService;
import cn.com.ctop.job.bytedance.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 图片与视频素材日报
 *
 * @author jeecg-boot
 * 2021-10-25
 * @version V1.0
 */
@Service
@Slf4j
public class BytedanceMaterialReportDailyServiceImpl extends ServiceImpl<BytedanceMaterialReportDailyMapper, BytedanceMaterialReportDaily> implements IBytedanceMaterialReportDailyService {
    @Value("${api.bytedance.api-prefix}")
    private String bytedanceApiUrl;
    @Autowired
    DataMapper dataMapper;

    @Override
    public void bytedanceDailyMaterialReportJob(OauthToken oauthToken, String yesterda, String yesterda1, int page, int pageSize) {
        // 请求地址
        String url = bytedanceApiUrl + BytedanceConstant.BYTEDANCE_V2_REPORT_INTEGRATED_GET;
        JSONObject param = new JSONObject();
        param.put("advertiser_id", oauthToken.getAccountId());
        param.put("start_date", yesterda1);
        param.put("end_date", yesterda1);
        param.put("page", page);
        param.put("page_size", pageSize);
        param.put("group_by", new String[]{"STAT_GROUP_BY_MATERIAL_ID", "STAT_GROUP_BY_INVENTORY", "STAT_GROUP_BY_IMAGE_MODE", "STAT_GROUP_BY_TIME_DAY"});

        JSONObject resultObject = HttpUtils.bytedanceGetRequest(oauthToken.getAccessToken(), url, param);

        Integer code = resultObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取素材日报接口异常==》accountId:{},message:{}", oauthToken.getAccountId(), resultObject.getString("message"));
            return;
        }
        JSONArray data = resultObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            return;
        }

        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            JSONObject dimensions = data.getJSONObject(i).getJSONObject("dimensions");
            JSONObject metrics = data.getJSONObject(i).getJSONObject("metrics");

            try {
                BytedanceMaterialReportDaily report = JSONObject.parseObject(metrics.toJSONString(), BytedanceMaterialReportDaily.class);
                report.setAdvertiserId(oauthToken.getAccountId());
                report.setMaterialId((Long) dimensions.get("material_id"));
                report.setImageMode((String) dimensions.get("image_mode"));
                report.setInventory((String) dimensions.get("inventory"));
                report.setStatDatetime(yesterda.replace("-", ""));
                report.setSubmitCertificationCount(dataObject.get("submit_certification_count")==null?null:Integer.parseInt((String) dataObject.get("submit_certification_count")));
                report.setApprovalCount(dataObject.get("approval_count")==null?null:Integer.parseInt((String) dataObject.get("approval_count")));
                report.setFirstOrderCount(dataObject.get("first_order_count")==null?null:Integer.parseInt((String) dataObject.get("first_order_count")));
                report.setFirstRentalOrderCount(dataObject.get("first_rental_order_count")==null?null:Integer.parseInt((String) dataObject.get("first_rental_order_count")));
                report.setCommuteFirstPayCount(dataObject.get("commute_first_pay_count")==null?null:Integer.parseInt((String) dataObject.get("commute_first_pay_count")));
                BytedanceMaterialReportDaily materialReport = dataMapper.getMaterialReportByPrimaryKey(report.getAdvertiserId(), report.getMaterialId(), report.getImageMode(), report.getInventory(), report.getStatDatetime());
                if (materialReport != null && materialReport.getAdvertiserId() != null) {
                    UpdateWrapper<BytedanceMaterialReportDaily> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.eq("advertiser_id", report.getAdvertiserId())
                            .eq("material_id", report.getMaterialId())
                            .eq("image_mode", report.getImageMode())
                            .eq("inventory", report.getInventory())
                            .eq("stat_datetime", report.getStatDatetime());
                    this.update(report, updateWrapper);
                } else {
                    this.save(report);
                }
            } catch (Exception e) {
                log.info("错误:{}", e.toString());
            }


        }
        page++;
        int totalPage = resultObject.getJSONObject("data").getJSONObject("page_info").getInteger("total_page");
        if (page <= totalPage) {
            bytedanceDailyMaterialReportJob(oauthToken, yesterda, yesterda1, page, 1000);
        }
    }
}
