package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.constant.CtopAdConstant;
import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IByteDanceAdvertiserDataService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 广告主定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedanceAgentLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IByteDanceAdvertiserDataService advertiserDataService;


    /**
     * 广告主日报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAgentDailyReport")
    public void bytedanceAdvertiserDailyReport() throws Exception {

        List<Long> agentByTouTiao = tokenService.getAgentByTouTiao();
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        for (Long aLong : agentByTouTiao) {
            String token = tokenService.getByAccountId(Long.valueOf(aLong));
            OauthToken oauthToken = new OauthToken();
            oauthToken.setAccessToken(token);
            oauthToken.setAccountId(aLong);
            dailyExecutorService.submit(
                    new Runnable() {
                        @Override
                        public void run() {
                            advertiserDataService.getAgentReport(oauthToken, yesterda, yesterda, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
                        }
                    });
        }

    }

    /**
     * 计划时报
     *
     * @throws Exception
     */
    @XxlJob("bytedanceAgentHourlyReport")
    public void bytedanceAdvertiserHourlyReport() throws Exception {
        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        List<Long> agentByTouTiao = tokenService.getAgentByTouTiao();
        for (Long aLong : agentByTouTiao) {
            String token = tokenService.getByAccountId(Long.valueOf(aLong));
            OauthToken oauthToken = new OauthToken();
            oauthToken.setAccessToken(token);
            oauthToken.setAccountId(aLong);
            hourExecutorService.submit(
                    new Runnable() {
                        @Override
                        public void run() {
                            advertiserDataService.getAgentReport(oauthToken, nowDate, nowDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
                        }
                    });
        }




    }
}
