package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.data.entity.OauthToken;
import cn.com.ctop.job.bytedance.data.service.IBytedanceMaterialReportDailyService;
import cn.com.ctop.job.bytedance.data.service.IOauthTokenService;
import cn.com.ctop.job.bytedance.data.utils.Check;
import cn.com.ctop.job.bytedance.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 头条素材定时任务
 *
 * @throws Exception
 */
@Slf4j
@Component
public class BytedanceMaterialReportLoadJob {
    @Autowired
    private IOauthTokenService tokenService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IBytedanceMaterialReportDailyService materialReportDailyService;


    /**
     * 获取头条图片素材
     *
     * @throws Exception
     */
    @XxlJob("bytedanceDailyMaterialReportJob")
    public void bytedanceDailyMaterialReportJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }

        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        OauthToken oauthToken = new OauthToken();
        oauthToken.setAccessToken(token);
        oauthToken.setAccountId(Long.parseLong(param));
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        materialReportDailyService.bytedanceDailyMaterialReportJob(oauthToken, yesterda,yesterda,1, 1000);
                    }
                });

    }


}
